'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _dp_picasso = require('@dp/picasso');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var PM_painting = function (context, param) {
    context.sendMessage("modulePainting", "painting", param);
};
var PM_paintingTab = function (context, param) {
    context.sendMessage("modulePainting", "paintingTab", param);
};
var PM_paintingContainer = function (context, param) {
    context.sendMessage("modulePainting", "paintingContainer", param);
};
var PM_paintingModulesVC = function (context, param) {
    context.sendMessage("modulePainting", "paintingModulesVC", param);
};
var PM_paintingChildVC = function (context, param) {
    context.sendMessage("modulePainting", "paintingChildVC", param);
};
var PM_sendEvent = function (context, param) {
    context.sendMessage("modulePainting", "sendEvent", param);
};

var PM_ScrollToRow = function (context, param) {
    return new Promise(function (resolve) {
        context.sendMessage("moduleEvents", "scrollToRow", Object.assign({}, param, {
            success: function () {
                param.success && param.success();
                resolve();
            }
        }));
    });
};
var PM_ScrollToSection = function (context, param) {
    return new Promise(function (resolve) {
        context.sendMessage("moduleEvents", "scrollToSection", Object.assign({}, param, {
            success: function () {
                param.success && param.success();
                resolve();
            }
        }));
    });
};
var PM_ScrollToModule = function (context, param) {
    return new Promise(function (resolve) {
        context.sendMessage("moduleEvents", "scrollToModule", Object.assign({}, param, {
            success: function () {
                param.success && param.success();
                resolve();
            }
        }));
    });
};
var PM_ScrollToPosition = function (context, param) {
    return new Promise(function (resolve) {
        context.sendMessage("moduleEvents", "scrollToPosition", Object.assign({}, param, {
            success: function () {
                param.success && param.success();
                resolve();
            }
        }));
    });
};
var PM_SetAnchor = function (context, param) {
    context.sendMessage("moduleEvents", "setAnchor", param);
};
var PM_RegisterKeyboard = function (context, param) {
    context.sendMessage("moduleEvents", "registerKeyboard", param);
};
var PM_ScrollToTop = function (context, param) {
    context.sendMessage("moduleEvents", "scrollToTop", param);
};

var VCIdGenerator = /** @class */ (function () {
    function VCIdGenerator() {
        this.vcId = 1;
    }
    VCIdGenerator.prototype.generateVcId = function () {
        return this.vcId++;
    };
    return VCIdGenerator;
}());
var PicassoModuleController = /** @class */ (function (_super) {
    __extends(PicassoModuleController, _super);
    function PicassoModuleController() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.m_identifier = 0;
        _this.m_callbackId = 0;
        _this.m_viewClzInfos = {}; // vcId => {clz, config}
        _this.m_viewDatas = {}; // vcId => data
        _this.m_viewEvents = {}; //vcId => events
        _this.m_viewIdentifier = {}; //identifier => vcId
        _this.m_moduleKeys = {}; //key => vcId
        _this.m_viewClassToJsName = {};
        _this.vcId = -1;
        _this.moduleData = {};
        /**
         * 存放子VC，用于原生直接控制渲染，可与原生通过vcid传递
         *
         * @protected
         * @type {VC}
         * @memberof VC
         */
        _this.childrenVC = {}; // vcId => vc
        return _this;
    }
    PicassoModuleController.prototype.dispatchOnLoad = function () {
        this.moduleData = this.intentData.moduleData;
        this.onLoad();
    };
    PicassoModuleController.prototype.dispatchOnDestroy = function () {
        for (var key in this.childrenVC) {
            var vc = this.childrenVC[key];
            if (vc) {
                vc.onDestroy();
            }
        }
        this.onDestroy();
    };
    PicassoModuleController.prototype.onLoad = function () {
        _super.prototype.onLoad.call(this);
    };
    PicassoModuleController.prototype.onDestroy = function () { };
    PicassoModuleController.prototype.generateIdentifier = function () {
        return (this.m_identifier++).toString();
    };
    PicassoModuleController.prototype.generateVcId = function () {
        if (this.rootVC) {
            return this.rootVC.generateVcId();
        }
        if (!this.vcIdGenerator) {
            this.vcIdGenerator = new VCIdGenerator();
        }
        return this.vcIdGenerator.generateVcId();
    };
    PicassoModuleController.prototype.getFunctionName = function (func) {
        for (var item in this) {
            if (this[item] === func) {
                return item;
            }
        }
        return "";
    };
    PicassoModuleController.prototype.registerCallBack = function (func) {
        var cbName = "__fn_callback_" + this.m_callbackId++;
        this[cbName] = func;
        return cbName;
    };
    /**
     * 在与原生传递VC视图前，先调用该方法传入VC class,返回的vcId可以传递给native，native会根据vcId找到对应的VC渲染
     * ex:this.configVC(MyPopoverVC)
     * @protected
     * @param {new () => PicassoController} clz
     * @param {{ data?: any, options?: VCOption }} [config]
     * @memberof VC
     */
    PicassoModuleController.prototype.configVC = function (clz, config) {
        var _this = this;
        var vcId = this.generateVcId();
        var childVC = this.initChildVC(vcId, clz, config);
        childVC.needLayout = function () {
            _this.context.sendMessage("modulePainting", "childVCNeedLayout", {
                vcId: vcId
            });
        };
        childVC.sendEvent = function (method, data) {
            var callBackName = _this.m_viewEvents[vcId][method];
            if (callBackName) {
                _this[callBackName](data);
            }
        };
        childVC.callModuleMethod = function (method, data) {
            _this[method](data);
        };
        return vcId;
    };
    PicassoModuleController.prototype.configModule = function (clz, config) {
        var vcId = this.generateVcId();
        var rootVC = this.rootVC ? this.rootVC : this;
        rootVC.m_viewClzInfos[vcId] = { clz: clz, config: config };
        return vcId;
    };
    PicassoModuleController.prototype.initChildVC = function (vcId, clz, config) {
        var childVC = new clz();
        childVC.hostId = this.hostId;
        childVC.context = this.context;
        if (config) {
            childVC.intentData = config.data;
            childVC.options = config.options || {
                width: this.width,
                height: this.height
            };
        }
        else {
            childVC.options = { width: this.width, height: this.height };
        }
        var rootVC = this.rootVC ? this.rootVC : this;
        childVC.rootVC = rootVC;
        childVC.vcId = vcId;
        rootVC.childrenVC[vcId] = childVC;
        return childVC;
    };
    PicassoModuleController.prototype.getChildVC = function (vcId) {
        var vc = this.childrenVC[vcId];
        if (vc)
            return vc;
        var rootVC = this.rootVC ? this.rootVC : this;
        var clzInfo = rootVC.m_viewClzInfos[vcId];
        if (!clzInfo)
            return null;
        var clz = clzInfo.clz, config = clzInfo.config;
        return this.initChildVC(vcId, clz, config);
    };
    PicassoModuleController.prototype.findChildVC = function (vcId) {
        var rootVC = this.rootVC ? this.rootVC : this;
        return rootVC.childrenVC[vcId];
    };
    /**
     * Host调用，call ChildVC中的方法，如onLoad,onAppear等
     *
     * @private
     * @param {string} method
     * @param {*} args
     * @memberof VC
     */
    PicassoModuleController.prototype.callChildVCByNative = function (args) {
        var vc = this.getChildVC(args.__vcid__);
        if (vc && vc[args.__method__]) {
            vc[args.__method__](args.params);
        }
    };
    PicassoModuleController.prototype.syncCallChildVCByNative = function (args) {
        var vc = this.getChildVC(args.__vcid__);
        if (vc && vc[args.__method__]) {
            return vc[args.__method__](args.params);
        }
    };
    /**
     * Host调用，发起layout刷新子VC视图
     *
     * @private
     * @returns
     * @memberof VC
     */
    PicassoModuleController.prototype.dispatchChildLayoutByNative = function (args) {
        if (this.childrenVC[args.vcId]) {
            if (args.options) {
                // 更新options，把宽高塞进去
                this.childrenVC[args.vcId].options = args.options;
            }
            return this.childrenVC[args.vcId].dispatchLayoutByNative();
        }
        // log('找不到对应的childVC')
        return {};
    };
    PicassoModuleController.prototype.dispatchBatchChildLayoutByNative = function (params) {
        var _this = this;
        var vcIds = params.args
            .filter(function (value) {
            var vc = _this.childrenVC[value.vcId];
            if (vc) {
                vc.dispatchOnLoad();
                // 更新options，把宽高塞进去
                vc.options = value.options;
                return true;
            }
            return false;
        })
            .map(function (value) {
            return value.vcId;
        });
        var childVCInfoMap = {};
        this.innerLayoutChildVCs(vcIds, childVCInfoMap);
        var childVCInfoList = [];
        for (var key in childVCInfoMap) {
            if (childVCInfoMap.hasOwnProperty(key)) {
                var vcInfo = childVCInfoMap[key];
                childVCInfoList.push({ vcId: Number(key), info: vcInfo });
            }
        }
        return childVCInfoList;
    };
    PicassoModuleController.prototype.innerLayoutChildVCs = function (vcIds, childVCInfoMap) {
        var allNeedSizeToFitViews = [];
        var needRelayoutVCs = [];
        for (var _i = 0, vcIds_1 = vcIds; _i < vcIds_1.length; _i++) {
            var vcid = vcIds_1[_i];
            var childVC = this.childrenVC[vcid];
            if (!childVC) {
                continue;
            }
            var vcInfo = childVC.innerLayout();
            if (vcInfo instanceof Array) {
                allNeedSizeToFitViews.push.apply(allNeedSizeToFitViews, vcInfo);
                needRelayoutVCs.push(vcid);
            }
            else {
                childVCInfoMap[vcid] = vcInfo;
            }
        }
        if (needRelayoutVCs.length > 0) {
            var allSizeResult = nativeSizeToFitList(allNeedSizeToFitViews);
            for (var _a = 0, needRelayoutVCs_1 = needRelayoutVCs; _a < needRelayoutVCs_1.length; _a++) {
                var vcid = needRelayoutVCs_1[_a];
                var childVC = this.childrenVC[vcid];
                if (!childVC) {
                    continue;
                }
                childVC.updateSizeCache(allSizeResult);
            }
            return this.innerLayoutChildVCs(needRelayoutVCs, childVCInfoMap);
        }
    };
    /**
     * Host调用，分发View的Action消息
     * @param args id:来源View的ViewId，action:Action名称，param:当前Action的附属参数
     * @memberof VC
     */
    PicassoModuleController.prototype.dispatchActionByNative = function (args) {
        for (var _i = 0, _a = Object.keys(this.childrenVC); _i < _a.length; _i++) {
            var key = _a[_i];
            var vc = this.childrenVC[key];
            if (!vc) {
                continue;
            }
            var action = vc.dispatchActionByNative(args);
            if (action) {
                return;
            }
        }
    };
    PicassoModuleController.prototype.destroyChildVC = function (args) {
        var vc = this.childrenVC[args.vcId];
        if (vc) {
            delete this.childrenVC[vc.vcId];
        }
    };
    PicassoModuleController.prototype.scrollToRow = function (param) {
        var newParam = Object.assign(param, this.rootVC ? { vcId: this.vcId } : {});
        PM_ScrollToRow(this.context, newParam);
    };
    PicassoModuleController.prototype.scrollToSection = function (param) {
        var newParam = Object.assign(param, this.rootVC ? { vcId: this.vcId } : {});
        PM_ScrollToSection(this.context, newParam);
    };
    PicassoModuleController.prototype.scrollToModule = function (param) {
        var newParam = Object.assign(param, this.rootVC ? { vcId: this.vcId } : {});
        PM_ScrollToModule(this.context, newParam);
    };
    PicassoModuleController.prototype.scrollToPosition = function (param) {
        var newParam = Object.assign(param, this.rootVC ? { vcId: this.vcId } : {});
        PM_ScrollToPosition(this.context, newParam);
    };
    PicassoModuleController.prototype.setAnchor = function (param) {
        var newParam = Object.assign(param, this.rootVC ? { vcId: this.vcId } : {});
        PM_SetAnchor(this.context, newParam);
    };
    return PicassoModuleController;
}(_dp_picasso.PicassoController));
function isEqual(newVal, oldVal) {
    if (!newVal || !oldVal)
        return newVal === oldVal;
    if (newVal instanceof Array) {
        return (oldVal instanceof Array &&
            newVal.every(function (item, i) { return isEqual(item, oldVal[i]); }));
    }
    else if (typeof newVal === "object") {
        return (typeof oldVal === "object" &&
            Object.keys(newVal).every(function (k) { return isEqual(newVal[k], oldVal[k]); }));
    }
    else {
        return newVal === oldVal;
    }
}
function addCallbackToScrollCellInfo(pm, cellInfo) {
    var cbKey = "attachStatusChangedCallback";
    var attachViewInfo = cellInfo.attachViewInfo;
    if (attachViewInfo && attachViewInfo.viewClass) {
        attachViewInfo.context = Object.assign(attachViewInfo.context || {}, {
            triggered: false
        });
        // 强行在viewInfo里面悄悄塞了回调
        attachViewInfo[cbKey] = cbKey;
    }
}
function addCallbackToTabCellInfo(pm, cellInfo) {
    var cbKey = "tabSelectedStatusChangedCallback";
    if (cellInfo.viewInfos) {
        cellInfo.viewInfos.forEach(function (viewInfo, index) {
            if (viewInfo.viewClass) {
                viewInfo.context = Object.assign(viewInfo.context || {}, {
                    selected: false
                });
                // 强行在viewInfo里面悄悄塞了回调
                viewInfo[cbKey] = cbKey;
            }
        });
    }
}
function addCallbackToTabModuleInfo(pm, tabModuleInfo) {
    var cbKey = "tabSelectedStatusChangedCallback";
    if (tabModuleInfo.viewInfos) {
        tabModuleInfo.viewInfos.forEach(function (viewInfo, index) {
            if (viewInfo.viewClass) {
                viewInfo.context = Object.assign(viewInfo.context || {}, {
                    selected: false
                });
                // 强行在viewInfo里面悄悄塞了回调
                viewInfo[cbKey] = cbKey;
            }
        });
    }
}
function swapCallback(pm, func) {
    return function (data) {
        var vcId = data.data["vcId"];
        data.data = pm.m_viewDatas[vcId];
        func.call(pm, data);
    };
}
function diffViewInfo(pm, newVal, oldVal) {
    // not imported view at all
    if (!newVal ||
        !newVal.viewClass ||
        newVal.viewType !== 2 /* PicassoVCImportedView */)
        return newVal;
    // clone except some fields
    var ret = {};
    for (var key in newVal) {
        if (key === "viewClass" || key === "data")
            continue;
        ret[key] = newVal[key];
    }
    // determine vcId and save in data
    var vcId = oldVal &&
        oldVal.data &&
        oldVal.data.vcId &&
        newVal.identifier == oldVal.identifier &&
        isEqual(newVal.data, pm.m_viewDatas[oldVal.data.vcId])
        ? oldVal.data.vcId
        : pm.configVC(newVal.viewClass, {
            data: {
                viewData: newVal.data,
                viewContext: newVal.context
            }
            // options: {width, height} // 待native侧设置
        });
    ret.data = {
        vcId: vcId
    };
    pm.m_viewDatas[vcId] = newVal.data;
    pm.m_viewEvents[vcId] = newVal.events ? newVal.events : {};
    if (newVal.identifier) {
        pm.m_viewIdentifier[newVal.identifier] = vcId;
    }
    if (newVal["tabSelectedStatusChangedCallback"]) {
        ret["tabSelectedStatusChangedCallback"] = (pm).registerCallBack(function (ctx) {
            var vc = pm.findChildVC(vcId);
            if (vc && vc.viewContext) {
                Object.assign(vc.viewContext, { selected: ctx.selected });
            }
            vc.needLayout();
        });
    }
    if (newVal["attachStatusChangedCallback"]) {
        ret["attachStatusChangedCallback"] = (pm).registerCallBack(function (ctx) {
            var vc = pm.findChildVC(vcId);
            if (vc && vc.viewContext) {
                Object.assign(vc.viewContext, { triggered: ctx.triggered });
            }
            vc.needLayout();
        });
    }
    return ret;
}
function diffModuleInfo(pm, newVal, oldVal) {
    // not imported view at all
    if (!newVal)
        return newVal;
    // clone except some fields
    var ret = {};
    for (var key in newVal) {
        if (key === "moduleKeys" || key === "extraModuleKeys") {
            var newModuleList = [];
            for (var index = 0; index < newVal[key].length; index++) {
                var moduleList = newVal[key][index];
                var newModule = [];
                for (var indexj = 0; indexj < moduleList.length; indexj++) {
                    var module = moduleList[indexj];
                    if (typeof module === "string") {
                        newModule[indexj] = module;
                    }
                    else if (typeof module === "object" &&
                        module.moduleKey &&
                        module.moduleClass) {
                        var rootVC = pm.rootVC ? pm.rootVC : pm;
                        var vcId = pm instanceof PicassoScrollTabModule // 永远新建
                            ? pm.configModule(module.moduleClass, {
                                data: {
                                    moduleData: module.moduleData
                                }
                            })
                            : rootVC.m_moduleKeys[module.moduleKey] ||
                                pm.configModule(module.moduleClass, {
                                    data: {
                                        moduleData: module.moduleData
                                    }
                                });
                        rootVC.m_moduleKeys[module.moduleKey] = vcId;
                        var prototype = module.moduleClass.prototype;
                        var prefixHeader = "";
                        if (prototype instanceof PicassoModule) {
                            prefixHeader = "picasso";
                        }
                        else if (prototype instanceof PicassoContainerModule) {
                            prefixHeader = "picassocontainer";
                        }
                        else if (prototype instanceof PicassoAnchorTabModule) {
                            prefixHeader = "picassotab";
                        }
                        else if (prototype instanceof PicassoScrollTabModule) {
                            prefixHeader = "picassoscrolltab";
                        }
                        else {
                            _dp_picasso.log(module.moduleKey + "实例类型不匹配");
                            return ret;
                        }
                        newModule[indexj] =
                            "GCPicassoImportedModule/" +
                                prefixHeader +
                                "_" +
                                module.moduleKey +
                                "___" +
                                pm.hostId +
                                "___" +
                                vcId +
                                "___module";
                    }
                }
                newModuleList[index] = newModule;
            }
            ret[key] = newModuleList;
        }
        else {
            ret[key] = newVal[key];
        }
    }
    return ret;
}
function transform(pm, newVal, oldVal) {
    if (!newVal)
        return newVal;
    if (newVal instanceof Array) {
        var identifierToViewInfo_1 = {};
        oldVal &&
            oldVal.forEach(function (item) {
                if (typeof item.identifier !== "undefined") {
                    identifierToViewInfo_1[item.identifier] = item;
                }
            });
        return newVal.map(function (item) {
            return transform(pm, item, identifierToViewInfo_1[item.identifier]);
        });
    }
    else if (typeof newVal === "object") {
        var ret = {};
        for (var key in newVal) {
            if (key === "data" ||
                key === "viewClass" ||
                key === "moduleKeys" ||
                key === "extraModuleKeys") {
                // special optimization
                ret[key] = newVal[key];
                continue;
            }
            if (key === "attachViewInfo" &&
                pm instanceof PicassoModule &&
                newVal.type === 2 /* Scroll */) {
                addCallbackToScrollCellInfo(pm, newVal);
            }
            if (key === "viewInfos" &&
                pm instanceof PicassoModule &&
                newVal.type === 5 /* Tab */) {
                addCallbackToTabCellInfo(pm, newVal);
            }
            if (key === "viewInfos" && pm instanceof PicassoTabModule) {
                addCallbackToTabModuleInfo(pm, newVal);
            }
            if ((key === "didSelectCallback" ||
                key === "exposeCallback" ||
                key === "tapMaskCallback" ||
                key === "appearOnScreenCallback" ||
                key === "disappearFromScreenCallback") &&
                typeof newVal[key] === "function" &&
                pm instanceof PicassoModuleController &&
                newVal.viewClass) {
                ret[key] = transform(pm, swapCallback(pm, newVal[key]), oldVal && oldVal[key]);
            }
            else {
                ret[key] = transform(pm, newVal[key], oldVal && oldVal[key]);
            }
        }
        if (ret && ret.viewClass) {
            ret.viewType = 2 /* PicassoVCImportedView */;
            var name = ret.viewClass.name;
            var viewClasses = pm.m_viewClassToJsName[name] || [];
            var index = viewClasses.indexOf(ret.viewClass);
            if (index < 0) {
                ret.jsName = name + viewClasses.length;
                viewClasses.push(ret.viewClass);
            }
            else {
                ret.jsName = name + index;
            }
            pm.m_viewClassToJsName[name] = viewClasses;
            return diffViewInfo(pm, ret, oldVal);
        }
        else if (ret && (ret.moduleKeys || ret.extraModuleKeys)) {
            return diffModuleInfo(pm, ret, oldVal);
        }
        return ret;
    }
    else if (typeof newVal === "function" &&
        pm instanceof PicassoModuleController) {
        return pm.registerCallBack(newVal);
    }
    else {
        return newVal;
    }
}
var PicassoVCView = /** @class */ (function (_super) {
    __extends(PicassoVCView, _super);
    function PicassoVCView() {
        var _this = _super.call(this) || this;
        _this.viewData = {};
        _this.viewContext = {};
        _this.vcId = -1;
        return _this;
    }
    PicassoVCView.prototype.dispatchOnLoad = function () {
        this.viewData = this.intentData.viewData;
        this.viewContext = this.intentData.viewContext;
        _super.prototype.dispatchOnLoad.call(this);
    };
    PicassoVCView.prototype.onLoad = function () {
        _super.prototype.onLoad.call(this);
    };
    PicassoVCView.prototype.sendEvent = function (tag, data) {
        this.sendMsg({
            type: 0 /* SendEvent */,
            tag: tag,
            data: data
        });
    };
    PicassoVCView.prototype.callModuleMethod = function (methodName, data) {
        this.sendMsg({
            type: 1 /* CallModuleMethod */,
            methodName: methodName,
            data: data
        });
    };
    PicassoVCView.prototype.configVC = function (clz, config) {
        var _this = this;
        var childVC = new clz();
        childVC.hostId = this.hostId;
        childVC.context = this.context;
        if (config) {
            childVC.intentData = config.data;
            childVC.options = config.options || {
                width: this.width,
                height: this.height
            };
        }
        else {
            childVC.options = { width: this.width, height: this.height };
        }
        var vcId = -1;
        if (this.rootVC) {
            vcId = this.rootVC.generateVcId();
            childVC.rootVC = this.rootVC;
            childVC.vcId = vcId;
            this.rootVC.childrenVC[vcId] = childVC;
        }
        else {
            vcId = this.childrenId++;
            this.childrenVC[vcId] = childVC;
        }
        childVC.needLayout = function () {
            _this.context.sendMessage("vc", "needChildLayout", { vcId: vcId });
        };
        return vcId;
    };
    return PicassoVCView;
}(_dp_picasso.VC));
var PicassoModule = /** @class */ (function (_super) {
    __extends(PicassoModule, _super);
    function PicassoModule() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.pm_extraInfo = {};
        _this.pm_moduleName = "";
        _this.m_moduleInfo = {};
        return _this;
    }
    PicassoModule.prototype.dispatchOnRefresh = function () {
        this.onRefresh();
    };
    PicassoModule.prototype.dispatchOnModuleAppear = function (info) {
        this.onAppear(info);
    };
    PicassoModule.prototype.dispatchOnModuleDisappear = function (info) {
        this.onDisappear(info);
    };
    PicassoModule.prototype.dispatchOnPageAppear = function () {
        this.onPageAppear();
    };
    PicassoModule.prototype.dispatchOnPageDisappear = function () {
        this.onPageDisappear();
    };
    PicassoModule.prototype.dispatchOnPaintSuccess = function () {
        this.onPaintSuccess();
    };
    PicassoModule.prototype.dispatchRetryForLoadingFail = function () {
        this.retryForLoadingFail();
    };
    PicassoModule.prototype.dispatchNeedLoadMore = function () {
        this.needLoadMore();
    };
    PicassoModule.prototype.onRefresh = function () { };
    PicassoModule.prototype.onAppear = function (info) { };
    PicassoModule.prototype.onDisappear = function (info) { };
    PicassoModule.prototype.onPageAppear = function () { };
    PicassoModule.prototype.onPageDisappear = function () { };
    PicassoModule.prototype.onPaintSuccess = function () { };
    PicassoModule.prototype.retryForLoadingFail = function () { };
    PicassoModule.prototype.needLoadMore = function () { };
    PicassoModule.prototype.painting = function (moduleInfo) {
        this.m_moduleInfo = transform(this, moduleInfo, this.m_moduleInfo);
        if (this.rootVC) {
            PM_paintingChildVC(this.context, {
                vcId: this.vcId,
                data: this.m_moduleInfo
            });
        }
        else {
            PM_painting(this.context, this.m_moduleInfo);
        }
    };
    PicassoModule.prototype.paintingSection = function (sectionInfo) {
        this.painting({ sectionInfos: [sectionInfo] });
    };
    PicassoModule.prototype.paintingCell = function (cellInfo) {
        this.paintingSection({ cellInfos: [cellInfo] });
    };
    PicassoModule.prototype.sendEvent = function (viewSendEventInfo) {
        var vcId = this.m_viewIdentifier[viewSendEventInfo.identifier];
        var VC$$1 = this.findChildVC(vcId);
        if (VC$$1) {
            VC$$1[viewSendEventInfo.method](viewSendEventInfo.data);
        }
        else {
            PM_sendEvent(this.context, viewSendEventInfo);
        }
    };
    PicassoModule.prototype.setPMExtraInfo = function (extraInfo) {
        this.pm_extraInfo = extraInfo;
    };
    PicassoModule.prototype.setPMModuleName = function (moduleNameInfo) {
        this.pm_moduleName = moduleNameInfo.moduleName;
    };
    PicassoModule.prototype.registerKeyboard = function (param) {
        var newParam = Object.assign(param, this.rootVC ? { vcId: this.vcId } : {});
        PM_RegisterKeyboard(this.context, newParam);
    };
    return PicassoModule;
}(PicassoModuleController));
var PicassoContainerModule = /** @class */ (function (_super) {
    __extends(PicassoContainerModule, _super);
    function PicassoContainerModule() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.m_containerInfo = {}; // 上次transfrom之后的containerInfo
        return _this;
    }
    PicassoContainerModule.prototype.paintingContainerModule = function (containerModuleInfo) {
        this.m_containerInfo = transform(this, containerModuleInfo, this.m_containerInfo);
        if (this.rootVC) {
            PM_paintingChildVC(this.context, {
                vcId: this.vcId,
                data: this.m_containerInfo
            });
        }
        else {
            PM_paintingContainer(this.context, this.m_containerInfo);
        }
    };
    return PicassoContainerModule;
}(PicassoModuleController));
var PicassoTabModule = /** @class */ (function (_super) {
    __extends(PicassoTabModule, _super);
    function PicassoTabModule() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.m_tabModuleInfo = { tabConfigs: [] };
        return _this;
    }
    PicassoTabModule.prototype.paintingTabModule = function (tabModuleInfo) {
        this.m_tabModuleInfo = transform(this, tabModuleInfo, this.m_tabModuleInfo);
        if (this.rootVC) {
            PM_paintingChildVC(this.context, {
                vcId: this.vcId,
                data: this.m_tabModuleInfo
            });
        }
        else {
            PM_paintingTab(this.context, this.m_tabModuleInfo);
        }
    };
    return PicassoTabModule;
}(PicassoModuleController));
var PicassoAnchorTabModule = /** @class */ (function (_super) {
    __extends(PicassoAnchorTabModule, _super);
    function PicassoAnchorTabModule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return PicassoAnchorTabModule;
}(PicassoTabModule));
var PicassoScrollTabModule = /** @class */ (function (_super) {
    __extends(PicassoScrollTabModule, _super);
    function PicassoScrollTabModule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PicassoScrollTabModule.prototype.scrollToTop = function (type) {
        var param = Object.assign({ type: type }, this.rootVC ? { vcId: this.vcId } : {});
        PM_ScrollToTop(this.context, param);
    };
    return PicassoScrollTabModule;
}(PicassoTabModule));
var PicassoModulesViewController = /** @class */ (function (_super) {
    __extends(PicassoModulesViewController, _super);
    function PicassoModulesViewController() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.m_modulesVCInfo = {};
        return _this;
    }
    PicassoModulesViewController.prototype.dispatchOnRefresh = function () {
        this.onRefresh();
    };
    PicassoModulesViewController.prototype.dispatchOnVcAppear = function () {
        this.onAppear();
    };
    PicassoModulesViewController.prototype.dispatchOnVcDisappear = function () {
        this.onDisappear();
    };
    PicassoModulesViewController.prototype.onRefresh = function () { };
    PicassoModulesViewController.prototype.onAppear = function () { };
    PicassoModulesViewController.prototype.onDisappear = function () { };
    PicassoModulesViewController.prototype.painting = function (modulesVCInfo) {
        this.m_modulesVCInfo = transform(this, modulesVCInfo, this.m_modulesVCInfo);
        if (this.rootVC) {
            PM_paintingChildVC(this.context, {
                vcId: this.vcId,
                data: this.m_modulesVCInfo
            });
        }
        else {
            PM_paintingModulesVC(this.context, this.m_modulesVCInfo);
        }
    };
    return PicassoModulesViewController;
}(PicassoModuleController));

exports.VCIdGenerator = VCIdGenerator;
exports.PicassoModuleController = PicassoModuleController;
exports.PicassoVCView = PicassoVCView;
exports.PicassoModule = PicassoModule;
exports.PicassoContainerModule = PicassoContainerModule;
exports.PicassoTabModule = PicassoTabModule;
exports.PicassoAnchorTabModule = PicassoAnchorTabModule;
exports.PicassoScrollTabModule = PicassoScrollTabModule;
exports.PicassoModulesViewController = PicassoModulesViewController;
